use maud::{PreEscaped, html};

// TODO : refactor shell

/// Represents the HTML structure of a page shell, including the head, body class, and body content.
///
/// This structure is used to construct the overall HTML structure of a page, making it easier to generate consistent HTML pages dynamically.
pub struct Shell {
    /// The HTML content for the `<head>` section of the page.
    head: PreEscaped<String>,
    /// An optional class attribute for the `<body>` element.
    body_class: Option<String>,
    /// The HTML content for the static body portion.
    body_content: PreEscaped<String>,
}

impl Shell {
    /// Constructs a new `Shell` instance with the given head content, body content, and body class.
    ///
    /// # Arguments
    /// * `head` - The HTML content for the page's head.
    /// * `body_content` - The HTML content for the body of the page.
    /// * `body_class` - An optional class to apply to the `<body>` element.
    ///
    /// # Returns
    /// A `Shell` instance encapsulating the provided HTML content and attributes.
    #[must_use]
    pub const fn new(
        head: PreEscaped<String>,
        body_content: PreEscaped<String>,
        body_class: Option<String>,
    ) -> Self {
        Self {
            head,
            body_class,
            body_content,
        }
    }

    /// Renders the full HTML page using the shell structure, with additional content and a title.
    ///
    /// # Arguments
    /// * `content` - The additional HTML content to render inside the main content div.
    /// * `title` - The title of the page, rendered inside the `<title>` element.
    ///
    /// # Returns
    /// A `PreEscaped<String>` containing the full HTML page content.
    #[must_use]
    pub fn render(&self, content: PreEscaped<String>, title: &str) -> PreEscaped<String> {
        html! {
                html {
                    head {
                        title { (title) };
                        (self.head)
                    };
                    @if self.body_class.is_some() {
                    body class=(self.body_class.as_ref().unwrap()) {
                        (self.body_content);

                        div id="main_content" {
                            (content)
                        };
                    };
                } @else {
                    body {
                        (self.body_content);

                        div id="main_content" {
                            (content)
                        };
                    };
                }
            }
        }
    }
}
